@@ -158,8 +158,8 @@ def init_theta(h_mult, n_qubits, J_eff, degrees):
     for q in prange(n_qubits):
         J = J_eff[q]
         z = degrees[q]
-        abs_zJ = abs(z * J)
-        theta[q] = (np.pi if J > 0 else -np.pi) if abs_zJ < epsilon else np.arcsin(max(-1.0, min(1.0, h_mult / (z * J))))
+        zJ = z * J
+        theta[q] = ((np.pi if J > 0 else -np.pi) / 2) if abs(zJ) < epsilon else np.arcsin(max(-1.0, min(1.0, h_mult / zJ)))
 
     return theta
 
@@ -187,11 +187,8 @@ def init_thresholds(n_qubits):
 
 @njit
 def probability_by_hamming_weight(J, h, z, theta, t, n_bias):
-    if abs(J) < epsilon:
-        return np.full(n_bias, 1.0 / n_bias, dtype=np.float64)
-
-    ratio = max(1.0, min(-1.0, abs(h) / (z * J)))
-    theta_c = np.arcsin(ratio)
+    zJ = z * J
+    theta_c = ((np.pi if J > 0 else -np.pi) / 2) if abs(zJ) < epsilon else np.arcsin(max(-1.0, min(1.0, h / zJ)))
 
     p = (
         2.0 ** (abs(J / h) - 1.0)
@@ -207,7 +204,7 @@ def probability_by_hamming_weight(J, h, z, theta, t, n_bias):
         result *= factor
         bias[q] = result
 
-    if (result == 0.0) or np.isnan(result):
+    if (result == 0.0) or np.isnan(result) or np.isinf(result):
         print("[WARN]: probability_by_hamming_weight() went below maximum precision.")
 
     if J > 0.0:
@@ -221,7 +218,7 @@ def maxcut_hamming_cdf(n_qubits, J_func, degrees, quality, tot_t, h_mult):
     hamming_prob = init_thresholds(n_qubits)
 
     n_steps = 1 << quality
-    delta_t = 1.0 / n_steps
+    delta_t = tot_t / n_steps
     n_bias = n_qubits + 1
 
     theta = init_theta(h_mult, n_qubits, J_func, degrees)
@@ -230,6 +227,8 @@ def maxcut_hamming_cdf(n_qubits, J_func, degrees, quality, tot_t, h_mult):
         step = qc // n_qubits
         q = qc % n_qubits
         J_eff = J_func[q]
+        if abs(J_eff) < epsilon:
+            continue
         z = degrees[q]
         theta_eff = theta[q]
         t = step * delta_t
@@ -256,3 +255,63 @@ def fix_cdf(hamming_prob):
     cum_prob[-1] = 2.0
 
     return cum_prob
+
+@njit
+def sample_mag(cum_prob):
+    p = np.random.random()
+    m = 0
+    left = 0
+    right = len(cum_prob) - 1
+    while True:
+        m = (left + right) >> 1
+
+        if (cum_prob[m] >= p) and ((m == 0) or cum_prob[m - 1] < p):
+            break
+
+        if cum_prob[m] < p:
+            left = m + 1
+        else:
+            right = m - 1
+
+    return m + 1
+
+@njit
+def init_bit_pick(weights, tot_init_weight, n):
+    r = np.random.rand()
+    cum = 0.0
+    node = 0
+    for i in range(n):
+        cum += weights[i]
+        if (tot_init_weight * r) < cum:
+            node = i
+            break
+
+    return node
+
+@njit
+def bit_pick(weights, used, n):
+    # Count available
+    total_w = 0.0
+    for i in range(n):
+        if used[i]:
+            continue
+        total_w += weights[i]
+
+    # Normalize & sample
+    r = np.random.rand()
+    cum = 0.0
+    node = -1
+    for i in range(n):
+        if used[i]:
+            continue
+        cum += weights[i]
+        if (total_w * r) < cum:
+            node = i
+            break
+
+    if node == -1:
+        node = 0
+        while used[node]:
+            node += 1
+
+    return node
